
#include <stdio.h>

int main()
{
  printf( "short = %d\n", sizeof(short) );
  printf( "int = %d\n",   sizeof(int)   );
  printf( "long = %d\n",  sizeof(long)  );
  printf( "void* = %d\n", sizeof(void*) );
  return 0;
}

